@@ -15,7 +15,7 @@ from mock_vws import MockVWS
 from mock_vws.database import VuforiaDatabase
 
 from vws import VWS, CloudRecoService
-from vws.exceptions import TargetProcessingTimeout
+from vws.exceptions.custom_exceptions import TargetProcessingTimeout
 from vws.reports import (
     DatabaseSummaryReport,
     TargetRecord,
@@ -57,7 +57,9 @@ class TestAddTarget:
             application_metadata=encoded_metadata,
             active_flag=active_flag,
         )
-        target_record = vws_client.get_target_record(target_id=target_id)
+        target_record = vws_client.get_target_record(
+            target_id=target_id,
+        ).target_record
         assert target_record.name == name
         assert target_record.width == width
         assert target_record.active_flag is active_flag
@@ -277,7 +279,8 @@ class TestGetTargetRecord:
             reco_rating='',
         )
 
-        assert result == expected_target_record
+        assert result.target_record == expected_target_record
+        assert result.status == TargetStatuses.PROCESSING
 
 
 class TestWaitForTargetProcessed:
@@ -333,18 +336,23 @@ class TestWaitForTargetProcessed:
             report = vws_client.get_database_summary_report()
             expected_requests = (
                 # Add target request
-                1 +
+                1
+                +
                 # Database summary request
-                1 +
+                1
+                +
                 # Initial request
-                1 +
+                1
+                +
                 # Request after 0.2 seconds - not processed
-                1 +
+                1
+                +
                 # Request after 0.4 seconds - not processed
                 # This assumes that there is less than 0.1 seconds taken
                 # between the start of the target processing and the start of
                 # waiting for the target to be processed.
-                1 +
+                1
+                +
                 # Request after 0.6 seconds - processed
                 1
             )
@@ -383,16 +391,20 @@ class TestWaitForTargetProcessed:
             report = vws_client.get_database_summary_report()
             expected_requests = (
                 # Add target request
-                1 +
+                1
+                +
                 # Database summary request
-                1 +
+                1
+                +
                 # Initial request
-                1 +
+                1
+                +
                 # Request after 0.3 seconds - not processed
                 # This assumes that there is less than 0.2 seconds taken
                 # between the start of the target processing and the start of
                 # waiting for the target to be processed.
-                1 +
+                1
+                +
                 # Request after 0.6 seconds - processed
                 1
             )
@@ -534,9 +546,9 @@ class TestUpdateTarget:
         )
 
         target_details = vws_client.get_target_record(target_id=target_id)
-        assert target_details.name == new_name
-        assert target_details.width == new_width
-        assert not target_details.active_flag
+        assert target_details.target_record.name == new_name
+        assert target_details.target_record.width == new_width
+        assert not target_details.target_record.active_flag
 
     def test_no_fields_given(
         self,
